from omegaconf import OmegaConf

# Import this file to register the resolvers with OmegaConf
__all__ = ()


# Register eq and neq checkers
OmegaConf.register_new_resolver("eq", lambda var, static_val: var == static_val)
OmegaConf.register_new_resolver("neq", lambda var, static_val: var != static_val)



import pytest
from airflow_balancer.testing import pools
from airflow_pydantic.migration import _airflow_3

from airflow_config import load_config


def test_config_and_options():
    with pools():
        conf = load_config("config", "complex")

        assert str(conf.model_dump_json(exclude_unset=True, indent=2, serialize_as_any=True))


def test_config_null_schedule():
    with pools():
        conf = load_config("config", "complex")

        assert "none-schedule" in conf.dags
        assert conf.dags["none-schedule"].schedule is None


def test_none_schedule_instantiation():
    try:
        from airflow.timetables.simple import NullTimetable
    except ImportError:
        pytest.skip("Airflow is not installed, skipping timetable tests")

    with pools():
        conf = load_config("config", "complex")

        assert "none-schedule" in conf.dags
        assert conf.dags["none-schedule"].schedule is None

        inst = conf.dags["none-schedule"].instantiate()
        if _airflow_3() or not hasattr(inst, "schedule_interval"):
            assert inst.schedule is None
        else:
            assert inst.schedule_interval is None
        assert isinstance(inst.timetable, NullTimetable)



from pathlib import Path
from unittest.mock import patch


class TestAirflowPlugin:
    def test_plugin(self, has_airflow_2):
        from airflow_config.ui.airflow import AirflowConfigViewerPlugin, AirflowConfigViewerPluginView

        AirflowConfigViewerPluginView()
        AirflowConfigViewerPlugin()


class TestPluginFunctions:
    def test_plugin_functions_get_yamls(self, has_airflow):
        from airflow_config.ui.functions import get_yaml_files

        root = Path(__file__).parent
        assert sorted(get_yaml_files(root)) == sorted(
            [
                root / "setups" / "priority" / "config" / "test.yaml",
                root / "setups" / "basic" / "config" / "basic.yaml",
                root / "setups" / "basic" / "config" / "basic2.yaml",
                root / "setups" / "basic" / "config" / "complex.yaml",
                root / "setups" / "basic" / "config" / "options.yaml",
                root / "setups" / "envs" / "config" / "dev.yaml",
                root / "setups" / "envs" / "config" / "prod.yaml",
                root / "setups" / "factory" / "config" / "balancer.yaml",
                root / "setups" / "factory" / "config" / "factory.yaml",
                root / "setups" / "factory" / "config" / "host.yaml",
                root / "setups" / "factory" / "config" / "multi.yaml",
                root / "setups" / "factory" / "config" / "self_reference.yaml",
                root / "setups" / "factory" / "config" / "templates.yaml",
                root / "setups" / "factory" / "config" / "type_override.yaml",
                root / "setups" / "specialize" / "config" / "disable.yaml",
                root / "setups" / "specialize" / "config" / "disable_env.yaml",
                root / "setups" / "specialize" / "config" / "disable_perdag.yaml",
                root / "setups" / "specialize" / "config" / "specialize.yaml",
            ]
        )

    def test_plugin_functions_get_configs(self, has_airflow):
        from airflow_config.ui.functions import get_configs_from_yaml

        root = Path(__file__).parent
        val = get_configs_from_yaml(Path(root) / "setups/basic/config/basic.yaml", [])
        print("\n\n")
        print(val)
        print("\n\n")
        assert val == '{\n  "default_task_args": {\n    "owner": "test"\n  },\n  "default_dag_args": {},\n  "dags": {}\n}'

    def test_plugin_functions_get_configs_backtick_bug(self, has_airflow):
        from airflow_config.ui.functions import get_configs_from_yaml

        root = Path(__file__).parent
        val = get_configs_from_yaml(Path(root) / "setups/factory/config/factory.yaml", [])
        print("\n\n")
        print(val)
        print("\n\n")

        assert (
            val
            == '{\n  "default_task_args": {\n    "owner": "test",\n    "email": [\n      "myemail@myemail.com"\n    ],\n    "email_on_failure": false,\n    "email_on_retry": false,\n    "retries": 0,\n    "depends_on_past": false\n  },\n  "default_dag_args": {\n    "schedule": "PT1H",\n    "start_date": "2024-01-01T00:00:00",\n    "catchup": false,\n    "tags": [\n      "utility",\n      "test"\n    ]\n  },\n  "dags": {\n    "example_dag": {\n      "description": "this is an example dag",\n      "schedule": "0 3 * * *",\n      "start_date": "2024-01-01T00:00:00",\n      "default_args": {\n        "owner": "custom_owner",\n        "email": [\n          "myemail@myemail.com"\n        ],\n        "email_on_failure": false,\n        "email_on_retry": false,\n        "retries": 0,\n        "depends_on_past": false\n      },\n      "catchup": false,\n      "tags": [\n        "utility",\n        "test"\n      ],\n      "dag_id": "example_dag",\n      "tasks": {\n        "task_1": {\n          "do_xcom_push": true,\n          "ssh_hook": {\n            "remote_host": "test",\n            "username": "test",\n            "port": 22,\n            "cmd_timeout": 10,\n            "keepalive_interval": 30,\n            "banner_timeout": 30.0\n          },\n          "ssh_conn_id": "test",\n          "command": "test",\n          "cmd_timeout": 10,\n          "environment": {\n            "test": "test"\n          },\n          "get_pty": true,\n          "task_id": "task_1"\n        },\n        "task_2": {\n          "python_callable": "airflow_config.tests.conftest.should_short_circuit",\n          "task_id": "task_2",\n          "dependencies": [\n            "task_1"\n          ]\n        },\n        "task_3": {\n          "bash_command": "echo \'1\'",\n          "task_id": "task_3",\n          "dependencies": [\n            "task_2"\n          ]\n        },\n        "task_4": {\n          "bash_command": "echo `pwd`",\n          "task_id": "task_4",\n          "dependencies": [\n            "task_3"\n          ]\n        },\n        "task_5": {\n          "python_callable": "airflow_config.tests.setups.utils.print_hello.print_hello",\n          "op_args": [],\n          "op_kwargs": {},\n          "templates_dict": {},\n          "templates_exts": null,\n          "show_return_value_in_logs": true,\n          "task_id": "task_5",\n          "dependencies": [\n            "task_4"\n          ]\n        }\n      }\n    },\n    "example_dag2": {\n      "schedule": "0 4 * * *",\n      "start_date": "2024-01-01T00:00:00",\n      "default_args": {\n        "owner": "custom_owner2",\n        "email": [\n          "myemail@myemail.com"\n        ],\n        "email_on_failure": false,\n        "email_on_retry": false,\n        "retries": 0,\n        "depends_on_past": false\n      },\n      "catchup": false,\n      "tags": [\n        "utility",\n        "test"\n      ],\n      "dag_id": "example_dag2",\n      "tasks": {\n        "task_1": {\n          "bash_command": "echo \\"1\\"",\n          "task_id": "task_1"\n        },\n        "task_2": {\n          "bash_command": "echo 2",\n          "task_id": "task_2",\n          "dependencies": [\n            "task_1"\n          ]\n        },\n        "task_3": {\n          "bash_command": "echo 3",\n          "task_id": "task_3",\n          "dependencies": [\n            "task_1"\n          ]\n        }\n      }\n    },\n    "example_dag3": {\n      "schedule": "0 5 * * *",\n      "start_date": "2024-01-01T00:00:00",\n      "default_args": {\n        "owner": "test",\n        "email": [\n          "myemail@myemail.com"\n        ],\n        "email_on_failure": false,\n        "email_on_retry": false,\n        "retries": 0,\n        "depends_on_past": false\n      },\n      "catchup": false,\n      "tags": [\n        "utility",\n        "test"\n      ],\n      "dag_id": "example_dag3"\n    }\n  }\n}'
        )


class TestStandaloneUI:
    def test_standalone_ui(self, has_airflow_2):
        from airflow_config.ui.standalone import build_app

        # Test the build_app function
        app = build_app()
        assert app is not None

    def test_launch(self, has_airflow_2):
        from airflow_config.ui.standalone import main

        # Test the main function
        with patch("airflow_config.ui.standalone.run") as mock_run:
            main()
            mock_run.assert_called_once()

    def test_main(self, has_airflow_2):
        import airflow_config.ui.standalone.__main__  # noqa: F401



import os
from pathlib import Path
from tempfile import TemporaryDirectory

import pytest
from airflow_pydantic.migration import _airflow_3
from jinja2 import DictLoader, Environment

from airflow_config import Configuration, DagArgs, TaskArgs


def should_short_circuit(**kwargs):
    return False


@pytest.fixture(autouse=True)
def airflow_config():
    config_template = (Path(__file__).parent / "airflow.cfg.jinja").read_text()
    j2 = Environment(loader=DictLoader({"airflow.cfg": config_template}), trim_blocks=True)
    with TemporaryDirectory() as td:
        tmpl = j2.get_template("airflow.cfg").render(
            # Set to root path of airflow_config
            DAGS_ROOT=str(Path(__file__).parent.parent.resolve()),
        )
        (Path(td) / "airflow.cfg").write_text(tmpl)
        os.environ["AIRFLOW_HOME"] = str(Path(td))
        os.environ["AIRFLOW_CONFIG"] = str((Path(td) / "airflow.cfg"))
        try:
            import airflow.configuration

            airflow.configuration.AIRFLOW_HOME = os.environ["AIRFLOW_HOME"]
            airflow.configuration.AIRFLOW_CONFIG = os.environ["AIRFLOW_CONFIG"]
            airflow_config_parser = airflow.configuration.AirflowConfigParser()
            airflow.configuration.load_standard_airflow_configuration(airflow_config_parser)
            airflow_config_parser.validate()
            airflow.configuration.conf = airflow_config_parser
        except ImportError:
            # Ignore
            ...
        yield


@pytest.fixture()
def has_airflow():
    try:
        from airflow.configuration import conf  # noqa: F401
    except ImportError:
        pytest.skip("Airflow is not installed, skipping tests")


@pytest.fixture()
def has_airflow_2():
    if _airflow_3() is not False:
        pytest.skip("Airflow 2 is not installed, skipping tests")


@pytest.fixture(scope="function", autouse=True)
def configuration() -> Configuration:
    config = Configuration(
        default_args=TaskArgs(),
        default_dag_args=DagArgs(),
        dags={},
    )
    return config



from functools import singledispatch
from inspect import currentframe
from typing import Any, Optional

from airflow_pydantic.airflow import DAG as BaseDag

from .configuration.base import Configuration
from .utils import generate_dag_id

__all__ = (
    "generate_dag_id",
    "create_dag",
    "create_dags",
    "DAG",
)


class DAG(BaseDag):
    def __init__(self, config: Configuration = None, **kwargs):
        if config:
            config.pre_apply(self, kwargs)
        super().__init__(**kwargs)
        if config:
            config.apply(self, kwargs)


@singledispatch
def create_dag(arg, **kwargs: Any) -> DAG:
    raise NotImplementedError()


@singledispatch
def create_dags(arg, **kwargs: Any) -> DAG:
    raise NotImplementedError()


@create_dag.register
def _create_dag_config(config: Configuration, dag_id: str = "", _offset: int = 3) -> DAG:
    dag_id = dag_id or generate_dag_id(offset=_offset + 1)
    dag = DAG(config=config, dag_id=dag_id)
    cur_frame = currentframe()
    for _ in range(_offset - 1):
        if hasattr(cur_frame, "f_back") and cur_frame.f_back and hasattr(cur_frame.f_back, "f_globals"):
            cur_frame = cur_frame.f_back
        else:
            break
    cur_frame.f_globals[dag_id] = dag
    return dag


@create_dag.register
def _create_dag_dir(config_dir: str = "config", config_name: str = "", overrides: Optional[list[str]] = None, dag_id: str = "", **kwargs: Any) -> DAG:
    dag_id = dag_id or generate_dag_id(offset=4)
    config = Configuration.load(config_dir=config_dir, config_name=config_name, overrides=overrides, _offset=5)
    return create_dag(config, dag_id=dag_id, _offset=5, **kwargs)


@create_dags.register
def _create_dags_config(configs: list, dag_ids: list[str] = None, **kwargs: Any) -> list[DAG]:
    ret = []
    dag_ids = dag_ids or []
    for i, config in enumerate(configs):
        if i < len(dag_ids):
            dag_id = dag_ids[i]
        else:
            dag_id = ""
        ret.append(create_dag(config, dag_id=dag_id, _offset=5, **kwargs))
    return ret


@create_dags.register
def _create_dags_dirs(
    config_dir: str = "config",
    config_names: list[list[str]] = None,
    overrides: Optional[list[str]] = None,
    dag_id_base: str = "",
    **kwargs: Any,
) -> list[DAG]:
    ret = []
    dag_id_base = dag_id_base or generate_dag_id(offset=4)
    for config_name in config_names:
        config = Configuration.load(config_dir=config_dir, config_name=config_name, overrides=overrides, _offset=5)
        ret.append(create_dag(config, dag_id=f"{dag_id_base}-{config_name}", _offset=5, **kwargs))
    return ret



__version__ = "1.11.0"

from .configuration import *

try:
    from .dag import DAG, create_dag, create_dags
except ImportError:
    # Allow airflow be removed
    pass
from .exceptions import *
from .utils import generate_dag_id



try:
    from airflow_priority.models import *  # noqa: F401, F403
except ImportError:
    pass

try:
    from airflow_balancer.config import *  # noqa: F401, F403
except ImportError:
    pass

from airflow_pydantic import *  # noqa: F401, F403

# NOTE: put this here to avoid airflow_balancer.load_config conflicting with this load_config
from .base import *  # noqa: F403



from configparser import ConfigParser
from inspect import currentframe
from os import environ, getcwd, path
from typing import cast
from uuid import uuid4

__all__ = ("generate_dag_id",)


def _get_calling_dag(offset: int = 2) -> str:
    cur_frame = currentframe()
    for _ in range(offset):
        if hasattr(cur_frame, "f_back") and cur_frame.f_back and hasattr(cur_frame.f_back, "f_globals"):
            cur_frame = cur_frame.f_back
        else:
            break
    if "__file__" not in cur_frame.f_globals:  # pragma: no cover
        # Assumed in ipython/interpreter
        return path.join(getcwd(), "ipython.py")  # pragma: no cover
    return cur_frame.f_globals["__file__"]


def _get_dag_root() -> str:
    home = environ.get("AIRFLOW_HOME", "")
    file = path.join(home, "airflow.cfg")
    if home and path.exists(file):
        config = ConfigParser()
        config.read(file)
        return config["core"]["dags_folder"]
    return ""


def generate_dag_id(name: str = "", dag_root: str = "", offset: int = 2) -> str:
    if not name:
        try:
            # get file of calling python file, can't use the module name as one might
            # have the same module in multiple folders
            caller = _get_calling_dag(offset=offset)
            # remove python suffix, replace path with dash
            name = caller.replace(".py", "").replace("/", "-").replace("_", "-")
            # remove root folder
            dag_root = dag_root or _get_dag_root()
            name = name.replace(dag_root.replace("/", "-").replace("_", "-"), "")
            while name.startswith("-"):
                name = name[1:]
        except Exception:
            name = cast(str, uuid4())
    return name



from pathlib import Path

import pytest


def test_generate_dag_id(airflow_config):
    try:
        from airflow_config import generate_dag_id
    except ImportError:
        pytest.skip("Airflow is not installed, skipping DAG tests")
    assert generate_dag_id(dag_root=str(Path(__file__).parent.parent.resolve())) == "tests-test-dag"


def test_create_dag_base(configuration):
    try:
        from airflow_config import create_dag
    except ImportError:
        pytest.skip("Airflow is not installed, skipping DAG tests")

    d = create_dag(configuration)
    assert d.dag_id == "tests-test-dag"
    assert "tests-test-dag" in globals()


def test_dag_base(configuration):
    try:
        from airflow_config import DAG
    except ImportError:
        pytest.skip("Airflow is not installed, skipping DAG tests")

    d = DAG(config=configuration, dag_id="test")
    assert d.dag_id == "test"
